/*
 * Gestion pompe piscine : program et manque d'eau
 * 
 * * Mesures de U et I  => deux entrées AN U sur A1 et I sur A0
 *                          Un sub qui lit  : int MesU[130] ; int MesI[130] ;
 *                          et qui retourne : int ValU; float ValI ; int ValDeph ;
 *                          
 * * Mesure de Température via Dallas DS18B20 => une E/S 1 wire pin 8              
 * 
 * * Gestion du relay => une sortie TOR sur 6
 * 
 * * Gestion BlueTooth => liaison série sup sur D10 RX et D11 TX et D7 sur EN à 0 bloque le BT
 *                        38400 baud
 * 
 * * Gestion du RTC => BUS I2C A4 SDA et A5 SCL pour DS1307
 * 
 * * Gestion du poussoir local de commande => une entrée Tor avec pull-up pin 2
 * 
 * * Gestion des Leds locales d'états => logic positive avec R de limitation
 * * Del VERTE 13
 * * Del BLEU 12
 * 
 * Luc BRUN 2019    
 */

// Date and time functions using a DS1307 RTC connected via I2C and Wire lib
#include <Wire.h>
#include "RTClib.h"
RTC_DS1307 rtc;
DateTime now;
char JourSem[7][12] = {"Dimanche", "Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi"};
unsigned long RTCTime, RTCFinModeManu,RTCFinBT, TimeEndError, FuturEnvoit ;
unsigned long TimePompeON;
byte HeurDebut1, HeurFin1, HeurDebut2, HeurFin2, HeurDebut3, HeurFin3 ; 

#include <SoftwareSerial.h> // BLUETOOTH HC-05
SoftwareSerial serialBluetooth(11, 10); // (TX du Bluetooth, RX du Bluetooth) 
const byte PinENBT = 7 ; boolean EtatBT = false;
String recep; char carac;
#include <OneWire.h> // Bus 1 Wire 
const byte BROCHE_ONEWIRE = 8; // ATTENTION résistance de 4k7 entre la borne et le +

enum DS18B20_RCODES { // Code de retour getTemperature()
  READ_OK,  // Lecture ok
  NO_SENSOR_FOUND,  // No capteur
  INVALID_ADDRESS,  // Adresse invalide
  INVALID_SENSOR  // Capteur invalide (pas un DS18B20)
};

OneWire ds(BROCHE_ONEWIRE); // Creation objet OneWire
float Temperature = -50; float NewTemperature = -50; byte TempIntern = 20;

const byte ledPin = 13; const byte ledBTPin = 12; // Led Verte & Led Bleu pour BT
volatile byte BTNPin = 2 ; // Pin bouton sur INT0
volatile byte BTNPress=0 ; volatile int TimePress;
byte cmt; // Compteur pour test ...

const byte PinU =  A1 ; const byte PinI = A0 ;   // entrées analogiques
const int ZeroU =  425 ; const int ZeroI = 511 ;   // zéros des entrées analogiques
const byte PinRelais = 6 ; 

int MesU[130] ; int MesI[130] ; int ValU; float ValI ; int ValDeph ;
long Umoy ; long Imoy ; int Umax ; int Imax ;int Umin ; int Imin ;
const byte TimRelOn = 32-20; const byte TimRelOff = 32-28;  // 32 = 10 ms

boolean TemperatureBasse = false ; boolean ERRORTemperature = false ;
boolean ModeError = false ; byte NumError = 0; boolean HorairePompage = false;
boolean EtatPompe = false; boolean ModeAuto = true;
boolean Bilan;

boolean SimulPompe = false ; boolean test = true;   // <-------ATTENTION TEST AND SIMULATION
//***************************************************************************
//***************************************************************************
byte getTemperature(float *temperature, byte reset_search) { // Lecture température DS18B20
  byte data[9], addr[8]; // Données lues depuis le scratchpad , Adresse du module 1-Wire détecté
  if (reset_search) {ds.reset_search();}   // Reset bus 1 Wire
  if (!ds.search(addr)) {return NO_SENSOR_FOUND;} // Pas de capteur
  if (OneWire::crc8(addr, 7) != addr[7]) {    return INVALID_ADDRESS;} // Adresse invalide
  if (addr[0] != 0x28) {return INVALID_SENSOR;  }  // Mauvais type de capteur (NO DS18B20)
  ds.reset();  ds.select(addr);   // Reset bus & sélectionne capteur
  ds.write(0x44, 1); delay(1000);   // Mesure de température et attend fin mesure
  ds.reset();  ds.select(addr);  ds.write(0xBE);   // Reset bus & sélectionne capteur & envoie demande lecture scratchpad
  for (byte i = 0; i < 9; i++) {data[i] = ds.read(); }  // Lecture du scratchpad
   
  *temperature = (int16_t) ((data[1] << 8) | data[0]) * 0.0625;   // Calcul de la température en degré Celsius
  return READ_OK;}
//***************************************************************************
//***************************************************************************
void MesureUID() {
  byte a,b,c ; unsigned long timing = millis();
  Umoy = 0; Imoy = 0; Umax = 0; Imax = 0; Umin = 32000; Imin = 32000;
  timing = millis();
  while (analogRead(PinU) > ZeroU && (millis()-timing ) < 100 )  {}
  while (analogRead(PinU) < ZeroU && (millis()-timing ) < 100 ) {}
  // U vient de passer positive en valeur decroissante

  //Serial.print(millis());Serial.print("--"); // Pour calibrer timing acquisition
  for ( int i = 0 ; i<128 ; i++) {
    MesU[i] = analogRead(PinU); MesI[i] = analogRead(PinI); 
    Umoy = Umoy + MesU[i]; if (MesU[i] > Umax) {Umax = MesU[i];} 
    if (MesU[i] < Umin) {Umin = MesU[i];}
    Imoy = Imoy + MesI[i]; if (MesI[i] > Imax) {Imax = MesI[i];}
    if (MesI[i] < Imin) {Imin = MesI[i];}
    // pause environ 1/64 de 20 ms soit 312 us
    delayMicroseconds(85); // A ajuster en fonction des calculs dans cette boucle
  }
  //Serial.println(millis());  // Pour calibrer timing acquisition

  if (SimulPompe) { // Pour test sans capteur fonctionnel :
    Umoy = 0; Imoy = 0; Umax = 0; Imax = 0;
    for ( int i = 0 ; i<128 ; i++) {
      MesU[i] = 511 + 312 * sin (i*0.098123);Umoy = Umoy + MesU[i];if (MesU[i] > Umax) {Umax = MesU[i];};if (MesU[i] < Umin) {Umin = MesU[i];}
      MesI[i] = 511 +  98 * sin (i*0.098123-0.87);Imoy = Imoy + MesI[i];if (MesI[i] > Imax) {Imax = MesI[i];};if (MesI[i] < Imin) {Imin = MesI[i];}}}
 
   Umoy = Umoy >> 7;  Imoy = Imoy >> 7; // Division par 128 mesures
  // il faut ramener Umoy et Imoy à 511 car à priori Umoy et Imoy sont entre 0 et 1023 normalement Umoy=0V ...
  for ( int i = 0 ; i<128 ; i++) {MesI[i] = MesI[i] + 511 - Imoy  ; MesU[i] = MesU[i] + 511 - Umoy ;}
  Umax = (Umax - Umin) >> 1; Imax = (Imax - Imin) >> 1; // valeurs crêtes

  //Il faut zoomer sur l'ondulation de I
  //for ( int i = 0 ; i<128 ; i++) {MesI[i] = 511 + 3 *(MesI[i] - 511) ; }
  //for ( int i = 0 ; i<128 ; i++) {MesI[i] = MesI[i] + MesI[i] + MesI[i] - 1022 ; } // Plus rapide
  for ( int i = 0 ; i<128 ; i++) {MesI[i] = MesI[i] + MesI[i] + MesI[i] + MesI[i] - 1533 ; } // Plus zoomé
  
  // Calcul des valeurs efficaces
  ValU = 231 ; ValI = Imax * 0.0336 ; // Calcul des valeurs efficaces à revoir pour i !!! RMS serait mieux
  
  // Calcul du déphasage
  a = 0;
  while (MesU[a] < Umoy  && a < 128) { a = a + 1 ;}
  while (MesU[a] > Umoy  && a < 128) { a = a + 1 ;}
  // a contient l'instant de passage de + à - pour U
  b = a ; if (b > 0) {b =b - 1;} 
  while (MesI[b] > Imoy  && b < 128) { b = b + 1 ;}
  ValDeph = 90; if (b < 125) {ValDeph = 5 + ((b-a) * 45 / 8) ;} // si phase > 35 le moteur est à vide ...

  //Recentrage de U et I entre 0 et 63 pour affichage avec une moyenne à 31 avec U en rouge I en jaune
  for ( int i = 0 ; i<128 ; i++) {MesI[i] = 63-(MesI[i] >> 4) ; MesU[i] = 63-(MesU[i] >> 4) ;} 

}
//***************************************************************************
//***************************************************************************
void PompeOnMesureUID() {
  byte a,b,c ; unsigned long timing = millis();
  Umoy = 0; Imoy = 0; Umax = 0; Imax = 0; Umin = 32000; Imin = 32000;
  timing = millis();
  while (analogRead(PinU) > ZeroU && (millis()-timing ) < 100 )  {}
  while (analogRead(PinU) < ZeroU && (millis()-timing ) < 100 ) {}

  for ( int i = 0 ; i<TimRelOn ; i++) { // Si TimRelOn = 32 = 10 ms
    MesU[i] = analogRead(PinU); MesI[i] = analogRead(PinI); 
    Umoy = Umoy + MesU[i]; if (MesU[i] > Umax) {Umax = MesU[i];} 
    if (MesU[i] < Umin) {Umin = MesU[i];}
    Imoy = Imoy + MesI[i]; if (MesI[i] > Imax) {Imax = MesI[i];}
    if (MesI[i] < Imin) {Imin = MesI[i];}
    delayMicroseconds(85);}
  
  digitalWrite(PinRelais, HIGH); if (test) {Serial.println(F("Pompe ON ..."));} // Relais ON
  for ( int i = TimRelOn ; i<128 ; i++) {
    MesU[i] = analogRead(PinU); MesI[i] = analogRead(PinI); 
    Umoy = Umoy + MesU[i]; if (MesU[i] > Umax) {Umax = MesU[i];} 
    if (MesU[i] < Umin) {Umin = MesU[i];}
    Imoy = Imoy + MesI[i]; if (MesI[i] > Imax) {Imax = MesI[i];}
    if (MesI[i] < Imin) {Imin = MesI[i];}
    delayMicroseconds(85);}
  
  Umoy = Umoy >> 7;  Imoy = Imoy >> 7; // Division par 128 mesures
  for ( int i = 0 ; i<128 ; i++) {MesI[i] = MesI[i] + 511 - Imoy  ; MesU[i] = MesU[i] + 511 - Umoy ;}
  Umax = (Umax - Umin) >> 1; Imax = (Imax - Imin) >> 1; // valeurs crêtes

  //  pas de Zoom sur l'ondulation de I car I démarrage
  
  //Recentrage de U et I entre 0 et 63 pour affichage avec une moyenne à 31 avec U en rouge I en jaune
  for ( int i = 0 ; i<128 ; i++) {MesI[i] = 63-(MesI[i] >> 4) ; MesU[i] = 63-(MesU[i] >> 4) ;} 
  if (EtatBT) {EnvoyerCourbes() ;}
}
//***************************************************************************
//***************************************************************************
void PompeOffMesureUID() {
  byte a,b,c ; unsigned long timing = millis();
  Umoy = 0; Imoy = 0; Umax = 0; Imax = 0; Umin = 32000; Imin = 32000;
  timing = millis();
  while (analogRead(PinI) > ZeroI && (millis()-timing ) < 100 )  {}   // Ici la synchro est sur le courant
  while (analogRead(PinI) < ZeroI && (millis()-timing ) < 100 ) {}

  for ( int i = 0 ; i<TimRelOff ; i++) {
    MesU[i] = analogRead(PinU); MesI[i] = analogRead(PinI); 
    Umoy = Umoy + MesU[i]; if (MesU[i] > Umax) {Umax = MesU[i];} 
    if (MesU[i] < Umin) {Umin = MesU[i];}
    Imoy = Imoy + MesI[i]; if (MesI[i] > Imax) {Imax = MesI[i];}
    if (MesI[i] < Imin) {Imin = MesI[i];}
    delayMicroseconds(85);}
  
  digitalWrite(PinRelais, LOW); if (test) {Serial.println(F("Pompe OFF ..."));} // Relais OFF
  for ( int i = TimRelOff ; i<128 ; i++) {
    MesU[i] = analogRead(PinU); MesI[i] = analogRead(PinI); 
    Umoy = Umoy + MesU[i]; if (MesU[i] > Umax) {Umax = MesU[i];} 
    if (MesU[i] < Umin) {Umin = MesU[i];}
    Imoy = Imoy + MesI[i]; if (MesI[i] > Imax) {Imax = MesI[i];}
    if (MesI[i] < Imin) {Imin = MesI[i];}
    delayMicroseconds(85);}
  
  Umoy = Umoy >> 7;  Imoy = Imoy >> 7; // Division par 128 mesures
  for ( int i = 0 ; i<128 ; i++) {MesI[i] = MesI[i] + 511 - Imoy  ; MesU[i] = MesU[i] + 511 - Umoy ;}
  Umax = (Umax - Umin) >> 1; Imax = (Imax - Imin) >> 1; // valeurs crêtes

  //for ( int i = 0 ; i<128 ; i++) {MesI[i] = MesI[i] + MesI[i] + MesI[i] - 1022 ; } //   Zoom sur l'ondulation de I
  for ( int i = 0 ; i<128 ; i++) {MesI[i] = MesI[i] + MesI[i] + MesI[i] + MesI[i] - 1533 ; } // Plus zoomé
  
  //Recentrage de U et I entre 0 et 63 pour affichage avec une moyenne à 31 avec U en rouge I en jaune
  for ( int i = 0 ; i<128 ; i++) {MesI[i] = 63-(MesI[i] >> 4) ; MesU[i] = 63-(MesU[i] >> 4) ;} 
  if (EtatBT) {EnvoyerCourbes() ;}
}
//***************************************************************************
//***************************************************************************
void InterruptionBTN() { // Gestion du poussoir appuis court & long
  int BTNHigh = 100;
  if (BTNPress != 0) {return;}
  TimePress = 0;  BTNPress = 1;
  while (BTNHigh > 0 && TimePress < 30000) {
    BTNHigh--;delayMicroseconds(100);
    while (digitalRead(BTNPin) == LOW && TimePress < 30000) { TimePress++;delayMicroseconds(200);}
    while (digitalRead(BTNPin) == HIGH && BTNHigh > 0) { BTNHigh--;delayMicroseconds(200);}
  }
  if (TimePress > 5000) {BTNPress = 2;}
  if (TimePress < 3) {BTNPress = 0;}
}
//***************************************************************************
//***************************************************************************
void PompeON() {
  TimePompeON = millis(); EtatPompe = true;
  if (SimulPompe) { digitalWrite(PinRelais, HIGH);if (test) {Serial.println(F("Pompe ON ..."));}}
  else {PompeOnMesureUID();}
}
//***************************************************************************
void PompeOFF() {
  EtatPompe = false;
  if (SimulPompe) { digitalWrite(PinRelais, LOW);if (test) {Serial.println(F("Pompe OFF ..."));}}
  else {PompeOffMesureUID();}
}
//***************************************************************************
//***************************************************************************
void EnvoyerCourbes() {
  byte Nbrpts = 32; byte decal;
  recep = "&C@" ; serialBluetooth.println(recep) ; delay (200); // Efface graph
  for (byte j=0; j < 4; j++) {
    decal = j * Nbrpts; recep = "&1," + String(decal) + "," ;       // &1 pour envoyer la courbe 1 : U
    for (byte i=0; i<Nbrpts;i++) {recep = recep + String (MesU[i+decal]) + "," ;}
    recep = recep + String (255)+ "@"; serialBluetooth.println(recep);}

  for (byte j=0; j < 4; j++) {
    decal = j * Nbrpts; recep = "&2," + String(decal) + "," ;       // 2 pour envoyer la courbe 2 : I
    for (byte i=0; i<Nbrpts;i++) {recep = recep + String (MesI[i+decal]) + "," ;}
    recep = recep + String (255)+ "@"; serialBluetooth.println(recep);}          
}          
//**************************************************************************************************
//**************************************************************************************************
unsigned long RTCLoadTime () {
  if (! rtc.isrunning()) {return 0;} // si erreur retourner 0 ...
  long result; now = rtc.now();
  result = ((now.year()-2000) << 11) + (now.month() << 5) + now.day(); result = result << 17;
  result = result + (now.hour() << 12) + (now.minute() << 6) + now.second();
  return result;}
//***************************************************************************
//***************************************************************************
float getInternalTemp(void){ // Mesure la température interne
  ADMUX = 0xC8;                // Sélectionne le port analogique 8 + référence interne 1v1
  ADCSRA |= (1 << ADEN);       // Active CAN
  delay (1000); uint16_t raw; uint32_t rawM = 0 ;
  for (byte i=1; i <= 10; i++){ // faire 10 fois pour plus de précision ...
    ADCSRA |= (1 << ADSC);       // Lance conversion AN
    while(ADCSRA & (1 << ADSC)); // WAIT END conversion
    raw = ADCL | (ADCH << 8); // résultat
    rawM = rawM + raw; delay (250);}
    
  rawM = rawM / 10;  
  return (0.5 + ((rawM - 324.31 ) / 1.22));     // Calcul la température
}
//***************************************************************************
//***************************************************************************
void setup() {
  recep.reserve(112) ;
  // Ouvre la voie série avec le module BT ATTENTION ICI 38400 BAUD (9600 par defaut)
  serialBluetooth.begin(38400);

  pinMode(ledPin, OUTPUT); delay (50); digitalWrite(ledPin, LOW); // Led Verte OFF
  pinMode(ledBTPin, OUTPUT); delay (50); digitalWrite(ledBTPin, LOW); // Led Bleu OFF
  
  pinMode(PinRelais, OUTPUT) ; delay (50); digitalWrite(PinRelais, LOW); // Relais OFF
  
  pinMode(PinENBT, OUTPUT) ; delay (50); digitalWrite(PinENBT, LOW); // BT OFF
  
  digitalWrite(ledPin, HIGH); delay (1000); digitalWrite(ledPin, LOW); 
  pinMode(BTNPin, INPUT_PULLUP) ; delay (50); // Bouton
  attachInterrupt(digitalPinToInterrupt (BTNPin), InterruptionBTN, FALLING); 
  delay (1000); BTNPress=0 ;

  //rtc.adjust(DateTime(F(__DATE__), F(__TIME__))); delay(2000);
  rtc.begin();
  
  test = true;
  if (test) {
    Serial.begin(9600);   Serial.println();  Serial.println(F("Hello !"));  
    if (! rtc.begin()) {Serial.println(F("Couldn't find RTC"));while (1);}
  }

  if (! rtc.isrunning()) {
    if (test) {Serial.println(F("RTC is NOT running!"));}
    rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
  }

  HeurDebut1=26; HeurFin1=26; HeurDebut2=26; HeurFin2=26; HeurDebut3=26; HeurFin3=26;
  BTNPress = 2; // Pour BlueTooth après Reset
}
//***************************************************************************
//***************************************************************************
void loop() {
  if (EtatPompe) { MesureUID ();} // mesures inutiles si pompe arrétée

  RTCTime = RTCLoadTime() ; // Lecture RTC
  if (!ModeError && (RTCTime == 0)) {ModeError= true ; NumError = 3;} // ERREUR RTC

  // Lecture température
  if (((millis()-TimePompeON) > 300000) && EtatPompe) { // NE FAIRE QUE SI LA POMPE TOURNE depuis 5 minutes !!!
    ERRORTemperature = false;
    if (getTemperature(&NewTemperature, true) != READ_OK) {
      Temperature=-50; ERRORTemperature = true; if (test) {Serial.println(F("Erreur de lecture Temperature")); }}
    else {
      if ((Temperature == -50) & (NewTemperature != -50)) {HorairePompage = false;}  // Pour mettre à jour RDVs
      Temperature = NewTemperature;}  
  }  

  //Surveillance du manque d'eau et du courant seulement si Pompe à ON, PAS mode Manu et démarré depuis plus de 10 s
  if (!ModeError && ModeAuto && EtatPompe && (millis()-TimePompeON) > 30000) { 
    if (ValI < 0.5) {ModeError= false ; NumError = 0;} // Le dephasage ne peut pas entrainer d'erreur ???
    else {
      if (ValDeph > 30) {ModeError= true ; NumError = 1;}    
      if (ValI > 4) {ModeError= true ; NumError = 2;}
    }
  }

  if (ModeError && (NumError != 3)) { // arret immédiat de la pompe jusqu'au prochain calcul d'horaire ...
    digitalWrite(PinRelais, LOW); EtatPompe = false; }

  if (HorairePompage && now.hour()<2) { // entre 0 et 1h on reset les horaires
    HorairePompage = false; ModeError= false ;}

  // ******* FIXE RDV ******************
  //Calcul RDV : fixer les horaires de pompage cela se fait une fois par jour apres 2h du mat !!!
  if (!HorairePompage && now.hour()>2) { // On fixe les horaires de pompages attention si RTC invalid !!!
    ModeError= false ; // Mise à zero du mode erreur critique
    HorairePompage = true; TemperatureBasse = false;
    EtatPompe = false; boolean ModeAuto = true; digitalWrite(PinRelais, LOW); // On est sur que la pompe est arretée
    
    if (Temperature < 4 && Temperature > -50) {TemperatureBasse = true;}  
    if (Temperature == -50) {ERRORTemperature = true;} // redondant ...   
    // Cas normal fixer HeurDebut1 HeurFin1 HeurDebut2 HeurFin2 HeurDebut3 HeurFin3 en fonction de Température
      HeurDebut1=26; HeurFin1=26; HeurDebut2=26; HeurFin2=26; HeurDebut3=26; HeurFin3=26;
      if (Temperature < 12) {HeurDebut1=9; HeurFin1=11;} // Filtrer 2H
      else if (Temperature < 15) {HeurDebut1=10; HeurFin1=11; HeurDebut2=14; HeurFin2=16;} // Filtrer 3H
      else if (Temperature < 18) {HeurDebut1=9; HeurFin1=12; HeurDebut2=15; HeurFin2=17;} // Filtrer 5H
      else if (Temperature < 21) {HeurDebut1=9; HeurFin1=12; HeurDebut2=14; HeurFin2=17;} // Filtrer 6H      
      else if (Temperature < 24) {HeurDebut1=9; HeurFin1=12; HeurDebut2=13; HeurFin2=17;} // Filtrer 7H      
      else if (Temperature < 27) {HeurDebut1=8; HeurFin1=12; HeurDebut2=13; HeurFin2=17;} // Filtrer 8H            
      else if (Temperature < 30) {HeurDebut1=8; HeurFin1=12; HeurDebut2=13; HeurFin2=18;} // Filtrer 9H                  
      else {HeurDebut1=9; HeurFin1=19;} // Filtrer 10H                  
  }
  // ******* END FIXE RDV ******************
  
  //***** RDV ******
  if (!ModeError && ModeAuto && HorairePompage) {   //gestion des rendez-vous de pompage
    if (ERRORTemperature) { // en cas d'erreur de température filtrer 6h de 10H à 16H
      if ((now.hour() >= 10) && (now.hour() < 16)) { // il faut filtrer
        if (!EtatPompe) {PompeON();}}
      else { // Il ne faut pas filtrer
        if (EtatPompe) { PompeOFF();}}
    }
    else if (TemperatureBasse) { // en cas de température très faible filter 10 mins toutes les 1/2 heures
      if ((now.minute() < 10) || ((now.minute() > 30) && (now.minute() < 40))) { // il faut filtrer
        if (!EtatPompe) { PompeON();}}
      else { // Il ne faut pas filtrer
        if (EtatPompe) { PompeOFF();}}
    }
    else { // Cas normal
      Bilan = (now.hour() >= HeurDebut1) && (now.hour() < HeurFin1) || (now.hour() >= HeurDebut2) && (now.hour() < HeurFin2) || (now.hour() >= HeurDebut3) && (now.hour() < HeurFin3);
      if (!EtatPompe && Bilan) {PompeON();} // il faut filtrer si pas déja en marche
      
      if (EtatPompe && !Bilan) {PompeOFF();} // il faut ARRETER filtrage si il est en marche
    }   
  }
  if (ModeError && (NumError == 3)) {  // Erreur RTC
    // Il faut filtrer 6H en se basant sur millis() A FAIRE <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
  }
  //***** END RDV ******
  
  //***** AUTOSTOP MODE MANU & STOP BLUETOOTH ******
  if (RTCTime > RTCFinModeManu && ModeAuto == false) {ModeAuto = true; PompeOFF(); if (test) {Serial.println(F("Passage en mode Auto ..."));}} // retourner en mode Automatique
  if (RTCTime > RTCFinBT && EtatBT == true) {EtatBT = false; pinMode(PinENBT, OUTPUT) ; delay (50); digitalWrite(PinENBT, LOW);if (test) {Serial.println(F("Arreter puce BT ..."));}} // stop BT 
  //***** END AUTOSTOP MODE MANU & STOP BLUETOOTH ******  

  //***** BOUTONS ******
  if (BTNPress == 1) { // Passer en mode manuel pour 1h
    BTNPress =0;
    ModeAuto = false; if (test) {Serial.println(F("Passage en mode Manu ..."));}
    if (!EtatPompe) { EtatPompe = true; PompeON() ; }
    else { EtatPompe = false; PompeOFF() ;}  
    RTCFinModeManu = RTCTime + 4096 ; // 64 * 64 car seconde et minutes codées sur 6 bits
  }

  if (BTNPress == 2) {
    BTNPress =0;
    if (EtatBT == false) {
      EtatBT = true; pinMode(PinENBT, INPUT) ;if (test) {Serial.println(F("BT ON ..."));}
      RTCFinBT = RTCTime + 640 ; // 10 * 64 car seconde et minutes codées sur 6 bits
      }
    else {EtatBT = false; pinMode(PinENBT, OUTPUT) ; delay (50); digitalWrite(PinENBT, LOW);
      if (test) {Serial.println(F("BT OFF ..."));}
      }
  }
  //***** END BOUTONS ******

   //*******IF BLUETOOTH ON*****************************
   if (EtatBT) {
    recep = ""; // reception d'un message BlueTooth
    while (serialBluetooth.available()) {carac = serialBluetooth.read(); recep = recep + String(carac); cmt=0;}
    if (recep != "" && test) {Serial.print("'");Serial.print(recep);Serial.println("'");}  
    if (recep != "") { // On a reçu un message 
      // &K demande Temperature interne suite à &K sur Android       &L demande les heures RDV
      // &Mmm fixe minutes &Hhh fixe heures
      // &C envoit courbes &1 et &2     &D envoit données ...
      // &A retour en mode AUTO    &B Force lecture Température et réévaluation RDV
      // &T &J &P &I &F 
      RTCFinBT = RTCTime + 640 ; // prolonger la durée du BT
      if (recep == "&C") { // Demande des courbes de U et I
        if (test) {Serial.println(F("Envoyer les courbes ..."));}
        EnvoyerCourbes() ;}  
      if (recep.substring(0,2) == "&H") { // Reception heure manuelle format &Hhh
        recep = recep.substring(2);
        rtc.adjust(DateTime(now.year(), now.month(), now.day(), recep.toInt(), now.minute(), now.second()));
        delay(1000);} 
      if (recep.substring(0,2) == "&M") { // Reception minute manuelle format &Mmm
        recep = recep.substring(2);
        rtc.adjust(DateTime(now.year(), now.month(), now.day(),now.hour() , recep.toInt(), now.second()));
        delay(1000);} 
      if (recep.substring(0,2) == "&J") { // Reception Date manuelle format &Jjj/mm/aaaa
        recep = recep.substring(2);
        rtc.adjust(DateTime(recep.substring(1+recep.lastIndexOf("/")).toInt(), recep.substring(1+recep.indexOf("/"),recep.lastIndexOf("/")).toInt(), recep.substring(0, recep.indexOf("/")).toInt(),now.hour() , now.minute(), now.second()));
        delay(1000);} 
      if (recep == "&P") { BTNPress = 1; } // equivalent appui court bouton
  
      if (recep == "&K") { // envoie de Température interne
        TempIntern = getInternalTemp(); analogReference(DEFAULT); delay (1000); 
        if (test) {Serial.print(F("Temperature Interne = ")); Serial.print(TempIntern); Serial.print(F("  U = ")); Serial.print(analogRead(PinU)); Serial.print(F("  I = ")); Serial.println(analogRead(PinI));}
        recep = "&K=" + String(TempIntern) + " @"; serialBluetooth.println(recep);delay (200);recep == "";}
  
      if (recep == "&L") { // envoie heures RDV
        recep = "&L=" + String(HeurDebut1) + " " + String(HeurFin1) + " "; 
        recep = recep + String(HeurDebut2) + " " + String(HeurFin2) + " "; 
        recep = recep + String(HeurDebut3) + " " + String(HeurFin3) + " @"; 
        serialBluetooth.println(recep);delay (200);recep == "";}
      if (recep == "&A") { // demande mode AUTO
        ModeAuto = true; PompeOFF(); delay (500);recep == "";}  
      if (recep == "&B") { // forcer mesure de Température
        ERRORTemperature = false; HorairePompage = false; ModeError= false ; // Pour mettre à jour RDVs
        if (getTemperature(&Temperature, true) != READ_OK) {
          Temperature=-50; ERRORTemperature = true; 
          if (test) {Serial.println(F("Erreur de lecture Temperature")); }}  
        delay (500);recep == "";}  
      if (recep == "&I") { // envoie de I
        recep = "&I" + String(ValI) + "@"; serialBluetooth.println(recep);delay (200);recep == "";}
      if (recep == "&T") { // envoie de Temperature
        recep = "&T" + String(Temperature)+ "@"; serialBluetooth.println(recep);delay (200);recep == "";}
      if (recep == "&F") { // envoie de la phase
        recep = "&F" + String(ValDeph)+ "@"; serialBluetooth.println(recep);delay (200);recep == "";}
      if (recep == "&D") { // Demande globale sans courbes
        //Serial.println(F("Envoyer les données ..."));
        recep = "&D" + String(now.day()) + "/" + String(now.month()) + " ";
        recep = recep + String(now.hour()) + ":" + String(now.minute())  + "@" ;
        serialBluetooth.println(recep);delay (100);recep == "";
        recep = "&I" + String(ValI) + "@"; serialBluetooth.println(recep);delay (100);recep == "";
        recep = "&T" + String(Temperature)+ "@"; serialBluetooth.println(recep);delay (100);recep == "";
        recep = "&F" + String(ValDeph)+ "@"; serialBluetooth.println(recep);delay (100);recep == "";
  
        if (ModeError == false) {
          recep = "&E" + String("Pompe OK")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";}
        else {
          if (NumError == 1) {recep = "&E" + String("ERR Deph") + "@";}
          else if (NumError == 2) {recep = "&E" + String("ERR I") + "@";}
          else if (NumError == 3) {recep = "&E" + String("ERR RTC")+ "@";}
          else {recep = "&E" + String("ERREUR ")+ String(NumError) + "@";}
          serialBluetooth.println(recep);delay (100);recep == "";} 
        if (ModeAuto) {recep = "&M" + String("Mode AUTO")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";}
        else { recep = "&M" + String("Mode Manu")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";}
        if (EtatPompe) {recep = "&P" + String("ON")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";}
        else { recep = "&P" + String("OFF")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";}
      }
    }
  
    if (FuturEnvoit < millis()) { // Envoit automatique des données
      recep = "&D" + String(now.day()) + "/" + String(now.month()) + " ";
      recep = recep + String(now.hour()) + ":" + String(now.minute())  + "@" ;
      serialBluetooth.println(recep);delay (100);recep == "";
  
      recep = "&I" + String(ValI) + "@"; serialBluetooth.println(recep);delay (100);recep == "";
      recep = "&T" + String(Temperature)+ "@"; serialBluetooth.println(recep);delay (100);recep == "";
      recep = "&F" + String(ValDeph)+ "@"; serialBluetooth.println(recep);delay (100);recep == "";
      // Affiche erreurs
      if (ModeError == false) {
          recep = "&E" + String("Pompe OK")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";}
      else {
        if (NumError == 1) {recep = "&E" + String("ERR Deph") + "@";}
        else if (NumError == 2) {recep = "&E" + String("ERR I") + "@";}
        else if (NumError == 3) {recep = "&E" + String("ERR RTC")+ "@";}
        else {recep = "&E" + String("ERREUR ")+ String(NumError) + "@";}
        serialBluetooth.println(recep);delay (100);recep == "";} 
      if (ModeAuto) {recep = "&M" + String("Mode AUTO")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";}
      else { recep = "&M" + String("Mode Manu")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";}
      if (EtatPompe) {recep = "&P" + String("ON")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";}
      else { recep = "&P" + String("OFF")+ "@"; serialBluetooth.println(recep);delay (100);recep == "";} 
      FuturEnvoit = millis() + 3000; // toute les 3 s environ ...
    }
  }
  //*******END IF BLUETOOTH ON*****************************
        

  //*******IF TEST ON**************************
  if (test) {
    Serial.println("");
    Serial.print(cmt++);Serial.print("  ");
    recep = String(now.day()) + "/" + String(now.month()) + "/" + String(now.year()) + "  "+ String(now.hour()) + ":" + String(now.minute()) + ":" + String(now.second());
    Serial.print(recep); 
    Serial.print(F("     Temperature = ")); Serial.print(Temperature, 1); Serial.write('C'); Serial.println(); // Affiche la température 
    Serial.print(F("Etat BT : ")); Serial.print(EtatBT);  Serial.print(F(" / Etat Pompe : ")); Serial.print(EtatPompe);Serial.print(F(" / ERREUR : ")); Serial.print(NumError); Serial.print(F(" / Mode Auto : ")); Serial.println(ModeAuto);}
  //*******END IF TEST ON**************************    

  
  //******** Gestion des deux leds *********
  if (!ModeError && !EtatBT && ModeAuto && EtatPompe) {digitalWrite(ledBTPin, LOW); digitalWrite(ledPin, HIGH); delay(1800); digitalWrite(ledPin, LOW); delay (1000); }

  if (!ModeError && !EtatBT && ModeAuto && !EtatPompe) {digitalWrite(ledBTPin, LOW); digitalWrite(ledPin, HIGH); delay(100); digitalWrite(ledPin, LOW); delay (2000);}

  if (!ModeError && !EtatBT && !ModeAuto && EtatPompe) {digitalWrite(ledBTPin, LOW); digitalWrite(ledPin, HIGH); delay(3000); digitalWrite(ledPin, LOW);delay (500);}

  if (!ModeError && !EtatBT && !ModeAuto && !EtatPompe) {digitalWrite(ledBTPin, LOW); digitalWrite(ledPin, HIGH); delay(500); digitalWrite(ledPin, LOW); delay (2000); }

  if (!ModeError && EtatBT && ModeAuto && EtatPompe) {digitalWrite(ledBTPin, HIGH); digitalWrite(ledPin, HIGH); delay(1800); digitalWrite(ledPin, LOW); digitalWrite(ledBTPin, LOW); delay (5000); }

  if (!ModeError && EtatBT && ModeAuto && !EtatPompe) {digitalWrite(ledBTPin, HIGH); digitalWrite(ledPin, HIGH); delay(100); digitalWrite(ledPin, LOW); digitalWrite(ledBTPin, LOW); delay (3000); }

  if (!ModeError && EtatBT && !ModeAuto && EtatPompe) {digitalWrite(ledBTPin, HIGH); digitalWrite(ledPin, HIGH); delay(2000); digitalWrite(ledPin, LOW); delay (1000); }

  if (!ModeError && EtatBT && !ModeAuto && !EtatPompe) {digitalWrite(ledBTPin, HIGH); digitalWrite(ledPin, HIGH); delay(1000); digitalWrite(ledPin, LOW); delay (1000); }

  if (ModeError) {digitalWrite(ledBTPin, HIGH); digitalWrite(ledPin, LOW); delay(2000); digitalWrite(ledPin, HIGH); digitalWrite(ledBTPin, LOW); delay(2000);}
  //******** END Gestion des deux leds *********
  
  
 
}
//***************************************************************************
